#!/usr/bin/env python

import math
#valid_nt=['A','C','G','T','U','Y','R','W','S','K','M','B','D','H','V','N']
valid_nt='ACGTUYRWSKMBDHVN'
#acgtuyrwskmbdhvn'

def nn_melting(seq,salt_cc,oligo_cc):
    # parameters from Santalucia, 1998 (http://www.pubmedcentral.gov/articlerender.fcgi?tool=pubmed&pubmedid=9465037)
    dHo_santalucia1998={'AA':-7.9,'TT':-7.9,'AT':-7.2,'TA':-7.2,'CA':-8.5,'TG':-8.5,'GT':-8.4,'AC':-8.4,'CT':-7.8,'AG':-7.8,'GA':-8.2,'TC':-8.2,'CG':-10.6,'GC':-9.8,'GG':-8.0,'CC':-8.0}
    dSo_santalucia1998={'AA':-22.2,'TT':-22.2,'AT':-20.4,'TA':-21.3,'CA':-22.7,'TG':-22.7,'GT':-22.4,'AC':-22.4,'CT':-21.0,'AG':-21.0,'GA':-22.2,'TC':-22.2,'CG':-27.2,'GC':-24.4,'GG':-19.9,'CC':-19.9}    

    seq=clear_seq(seq).upper()
    enth,entr=0,0

    for i in range(len(seq)-1):
        enth=enth+dHo_santalucia1998[seq[i]+seq[i+1]]
        entr=entr+dSo_santalucia1998[seq[i]+seq[i+1]]
       
    if seq[1] in ['A','T']:
        enth+=2.3
        entr+=4.1
        
    if seq[1] in ['C','G']:
        enth+=0.1
        entr+=-2.8

    if seq[-1] in ['A','T']:
        enth+=2.3
        entr+=4.1
        
    if seq[-1] in ['C','G']:
        enth+=0.1
        entr+=-2.8

    enth=enth*4.18*1000
    entr=entr*4.18
    
    tm=enth/(entr+0.368*(len(seq)-1)*math.log(salt_cc)*4.18+1.987*4.18*math.log(oligo_cc/4))-273.15
    
    return tm,enth,entr

#
def GCp(seq):
    count_gc,count_tot=0,0
    seq=seq.upper()
    for nt in seq:
        if nt in ['C','G','S','N']:
            count_gc+=1
        if nt in ['M','R','Y','K']:
            count_gc+=0.5
        if nt in ['V','B']:
            count_gc+=0.667
        if nt in ['H','D']:
            count_gc+=0.333
        if nt in valid_nt:
            count_tot+=1 
    return count_gc*1.0/count_tot;

#
def clear_seq(seq):
    seq2=''
    for nt in seq:
        if nt.upper() in valid_nt:
            seq2=seq2+nt
    return seq2        
#

def GCG_checksum(seq):
    seq=clear_seq(seq).upper()
    cs,count=0,0
    for nt in seq:
        if nt in valid_nt:
            count+=1
            cs=cs+count*ord(nt);
            if count==57:
                count=0
    cs=cs-(cs / 10000)*10000;

    return cs
#
def long_seq_melting(salt_cc,GCp,formamide_cc,seq_len):
    #formula from Howley, 1979
    return 81.5+16.6*math.log10(salt_cc)+0.41*GCp-0.62*formamide_cc-500/seq_len



#seq='AATCGTACGACTA'
#seq='125 aCTACGXAT G ATAGA'
#print clear_seq(seq)
#tm=nn_melting(seq,50e-3,250e-9)[0]
#print round(tm,2)," C"
#print GCp(seq)
#print GCG_checksum(seq)
#print long_seq_melting(1,57.16,0,229354)
